﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using Plclike;
using PlclikeClient;

namespace PlclikeClientTestCSherp
{
    /// <summary>
    /// PlclikeClient サンプルプログラム
    /// </summary>
    public partial class Form1 : Form
    {
        /// <summary>
        /// PlclikeClient 起動フォーム
        /// </summary>
        PlclikeClient.frmPLCSend fPlc;
        
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public Form1()
        {
            InitializeComponent();
        }

        #region "フォームイベント処理"

        /// <summary>
        /// FormClosingイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Plclike 起動フォームが生成されている場合
            if (fPlc != null)
            {
                // PlclikeClient 実行確認
                if( fPlc.RunMode== 1)
                {
                    // PlclikeClient 停止します。
                    fPlc.StopRun();
                }
                // 一応 Plclike 起動フォームを終了しておきます。
                fPlc.Close();
                fPlc.Dispose();
            }
        }

        /// <summary>
        /// FormLoadイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {

        }

        /// <summary>
        /// FormShownイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Shown(object sender, EventArgs e)
        {
            // Plclike起動状態の表示
            DispRunStatus(0);
        }

        #endregion

        #region "コントロールイベント処理"

        /// <summary>
        /// PlclikeClient 起動処理ボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StartButton_Click(object sender, EventArgs e)
        {
            // Plclike 起動処理
            if (StartPlclike() == false)
            {
                return;
            }

            // Plclike起動状態の表示
            DispRunStatus(0);
        }

        /// <summary>
        /// PlclikeClient 実行・停止ボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void runStopButton_Click(object sender, EventArgs e)
        {
            int iRes = -1;
            string message = "";

            // PLC通信情報の有無
            if (fPlc == null)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }
            else if (fPlc.IsDisposed == true)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }

            // 起動／実行確認
            if (fPlc.RunMode == 1)
            {
                // 通信切断開始
                iRes = fPlc.StopRun();
                if (iRes == 0)
                {
                    message = "PLC通信を停止しました。";
                    AddLoggingMessage(message);
                }
                else
                {
                    message = "PLC通信を停止時にエラーが発生しました。" + fPlc.ErrorMessage;
                    AddLoggingMessage(message);
                }
            }
            else
            {
                // 値設定情報が作成されていないようなら設定する
                if ( this.comboDevice.Items.Count <= 0 )
                {
                    // 設定情報取得
                    SetCommboBoxDevice(true);
                    this.textAddress.Text = "0000";
                    this.textValue.Text = "0000";
                    this.textAddress2.Text = "0000";
                    this.textValue2.Text = "0000";
                }

                // 通信接続開始
                iRes = fPlc.StartRun();
                if (iRes == 0)
                {
                    message = "PLC通信を実行しました。";
                    AddLoggingMessage(message);
                }
                else
                {
                    message = "PLC通信を実行時にエラーが発生しました。" + fPlc.ErrorMessage;
                    AddLoggingMessage(message);
                }
            }

            DispRunStatus(0);
        }

        /// <summary>
        /// メモリ読出しボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MemoryReadButton_Click(object sender, EventArgs e)
        {
            string sDevice = this.comboDevice.Text;
            string sAddress = this.textAddress.Text;
            string message = "";

            // PLC通信情報の有無
            if (fPlc == null)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }
            else if (fPlc.IsDisposed == true)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }

            // 読出し処理(イベント返信処理となります)
            int iRes = fPlc.SendRead(this.comboDevice.SelectedIndex, this.textAddress.Text, (int)this.numCh.Value);
            if (iRes < 0)
            {
                message += "読出送信エラー：" + fPlc.ErrorMessage;
                AddLoggingMessage(message);
            }

        }

        /// <summary>
        /// メモリ読出しボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MemoryReadButton2_Click(object sender, EventArgs e)
        {
            string sDevice = this.comboDevice.Text;
            string sAddress = this.textAddress.Text;
            string sValue = "";
            string message = "";

            // PLC通信情報の有無
            if (fPlc == null)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }
            else if (fPlc.IsDisposed == true)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }

            // 読出し処理（返信があるまで処理は戻ってきません。）
            int iRes = fPlc.SendRead(this.comboDevice.SelectedIndex, this.textAddress.Text, (int)this.numCh.Value, out sValue);
            if (iRes < 0)
            {
                message += "読出送信エラー：" + fPlc.ErrorMessage;
                AddLoggingMessage(message);
            }
            else
            {
                // 設定値代入
                textValue.Text = sValue;

                message += "読出送信：" + sValue;
                AddLoggingMessage(message);
            }
        }

        /// <summary>
        /// メモリ書込みボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MemoryWriteButton_Click(object sender, EventArgs e)
        {
            string sDevice = this.comboDevice2.Text;
            string sAddress = this.textAddress2.Text;
            string sValue = this.textValue2.Text;
            string message = "";
            int iCh = (int)this.numCh2.Value;
            int iRes = -1;

            // PLC通信情報の有無
            if (fPlc == null)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }
            else if (fPlc.IsDisposed == true)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }
            // 書込み値調整
            sValue = sValue.PadLeft((iCh * 4), '0');
            if (sValue.Length > (iCh * 4))
            {
                sValue = sValue.Substring(0, (iCh * 4));
            }
            if (this.textValue2.Text.CompareTo(sValue) != 0)
            {
                this.textValue2.Text = sValue;
            }

            // 書込み処理
            iRes = fPlc.SendWrite(this.comboDevice2.SelectedIndex, this.textAddress2.Text, iCh, sValue);
            if (iRes < 0)
            {
                message += "書込送信エラー：" + fPlc.ErrorMessage;
                AddLoggingMessage(message);
            }
        }

        /// <summary>
        /// メモリ書込みボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MemoryWriteButton2_Click(object sender, EventArgs e)
        {
            string sDevice = this.comboDevice2.Text;
            string sAddress = this.textAddress2.Text;
            string sValue = this.textValue2.Text;
            string message = "";
            string sResponse = "";
            int iLen = 0;
            int iCh = (int)this.numCh2.Value;
            int iRes = -1;

            // PLC通信情報の有無
            if (fPlc == null)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }
            else if (fPlc.IsDisposed == true)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }

            // 書込み値調整
            sValue = sValue.PadLeft((iCh * 4), '0');
            if(sValue.Length > (iCh*4))
            {
                sValue = sValue.Substring(0, (iCh * 4));
            }
            if ( this.textValue2.Text.CompareTo(sValue)!= 0)
            {
                this.textValue2.Text = sValue;
            }

            // 書込み処理
            iRes = fPlc.SendWrite(this.comboDevice2.SelectedIndex, this.textAddress2.Text, iCh, sValue, out sResponse);
            if (iRes < 0)
            {
                // 書込みエラー
                message += "書込送信エラー：" + sResponse + "：" + fPlc.ErrorMessage;
                AddLoggingMessage(message);
            }
            else
            {
                // 書込み成功
                if (String.IsNullOrEmpty(fPlc.ErrorMessage) == false)
                {
                    sValue = fPlc.ErrorMessage;
                }
                message += "書込送信：" + sResponse + "：" + sValue;
                AddLoggingMessage(message);
            }
        }

        /// <summary>
        /// 読取設定値入力ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnADDRESS1_INPUT_Click(object sender, EventArgs e)
        {
            Plclike.Libs.clsCommon cCommon = new Plclike.Libs.clsCommon();
            int value = 0;
            if (fPlc != null)
            {
                if (int.TryParse(this.textAddress.Text, out value) == false)
                {
                    value = 0;
                }
                int defaultvalue = value;
                // 設定値入力
                // 入力範囲：-2,147,483,648 ～ 2,147,483,647
                string sTitle = "アドレスの入力";
                int iRes = fPlc.InputNumeric(-2147483648, 2147483647, defaultvalue, sTitle, out value);
                if (iRes == 0)
                {
                    this.textAddress.Text = value.ToString().PadLeft(4, '0');
                }
            }
        }

        /// <summary>
        /// 読取設定値入力ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnVALUE1_INPUT_Click(object sender, EventArgs e)
        {
            Plclike.Libs.clsCommon cCommon = new Plclike.Libs.clsCommon();
            int value = 0;
            if (fPlc != null)
            {
                value = cCommon.Conv16to10(this.textValue.Text);
                int defaultvalue = value;
                // 設定値入力
                // 入力範囲：-2,147,483,648 ～ 2,147,483,647
                string sTitle = "設定値(16進数)の入力";
                int iRes = fPlc.InputHex(-2147483648, 2147483647, defaultvalue, sTitle, out value);
                if (iRes == 0)
                {
                    this.textValue.Text = cCommon.Conv10to16(value).ToUpper().PadLeft(4, '0');
                }
            }
        }

        /// <summary>
        /// 書込アドレス入力ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnADDRESS2_INPUT_Click(object sender, EventArgs e)
        {
            Plclike.Libs.clsCommon cCommon = new Plclike.Libs.clsCommon();
            int value = 0;
            if (fPlc != null)
            {
                if (int.TryParse(this.textAddress2.Text, out value) == false)
                {
                    value = 0;
                }
                int defaultvalue = value;
                // 設定値入力
                // 入力範囲：-2,147,483,648 ～ 2,147,483,647
                string sTitle = "アドレスの入力";
                int iRes = fPlc.InputNumeric(-2147483648, 2147483647, defaultvalue, sTitle, out value);
                if (iRes == 0)
                {
                    this.textAddress2.Text = value.ToString().PadLeft(4, '0');
                }
            }
        }

        /// <summary>
        /// 書込設定値入力ボタンクリックイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnVALUE2_INPUT_Click(object sender, EventArgs e)
        {
            Plclike.Libs.clsCommon cCommon = new Plclike.Libs.clsCommon();
            int value = 0;
            if (fPlc != null)
            {
                value = cCommon.Conv16to10(this.textValue2.Text);
                int defaultvalue = value;
                // 設定値入力
                // 入力範囲：-2,147,483,648 ～ 2,147,483,647
                string sTitle = "設定値(16進数)の入力";
                int iRes = fPlc.InputHex(-2147483648, 2147483647, defaultvalue, sTitle, out value);
                if (iRes == 0)
                {
                    this.textValue2.Text = cCommon.Conv10to16(value).ToUpper().PadLeft(4, '0');
                }
            }
        }

        /// <summary>
        /// 通信設定画面起動
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void configButton_Click(object sender, EventArgs e)
        {
            bool bRes = ConfigPlclike();
            if (bRes == false)
            {
                return;
            }
        }

        /// <summary>
        /// 処理内容のクリアボタンイベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MessageClearButton_Click(object sender, EventArgs e)
        {
            // 処理内容クリア処理
            ClrLoggingMessage();
        }

        #endregion

        #region "内部処理"

        /// <summary>
        /// PlclikeClient 実行状態取得
        /// </summary>
        /// <returns>0=停止、1=実行、それ以外=未定義</returns>
        private int GetRunStatus()
        {
            int iRes = -1;
            string sMessage = "";
            try
            {
                if (fPlc == null)
                {
                    // 未定義
                    iRes = -1;
                }
                else if (fPlc.IsDisposed == true)
                {
                    // 未定義
                    iRes = -1;
                }
                else if (fPlc.RunMode == 1)
                {
                    // 起動中
                    iRes = 1;
                }
                else
                {
                    // 停止中
                    iRes = 0;
                }
            }
            catch(Exception e)
            {
                // 例外エラー
                iRes = -2;
                sMessage = e.Message;
            }
            return iRes;
        }

        /// <summary>
        /// PlclikeClient 実行状態取得
        /// </summary>
        /// <returns>状態情報</returns>
        private string GetRunStatusName()
        {
            string sMessage = "";
            try
            {
                if (fPlc == null)
                {
                    sMessage = "未処理";
                }
                else if (fPlc.IsDisposed == true)
                {
                    sMessage = "未処理";
                }
                else if (fPlc.RunMode == 1)
                {
                    sMessage = "起動中";
                }
                else
                {
                    sMessage = "停止中";
                }
            }
            catch(Exception e)
            {
                sMessage = "例外エラー：" + e.Message;
            }
            return sMessage;
        }

        /// <summary>
        /// PlclikeClient 起動状態
        /// </summary>
        /// <param name="iFlag">0=Open/Closes処理時,1=Dispose処理時</param>
        private void DispRunStatus(int iFlag)
        {
            // 接続状態表示
            string sMessage = GetRunStatusName();
            this.lblRunStatus.Text = "PlclikeClientの状態：" + sMessage;
            // 起動・停止ボタン表示切替
            if (GetRunStatus() == 1)
            {
                this.runStopButton.Text = "接続停止";
                this.runStopButton.ForeColor = Color.Red;
            }
            else
            {
                this.runStopButton.Text = "実行開始";
                this.runStopButton.ForeColor = Color.Blue;
            }
        }

        /// <summary>
        /// PlclikeClient 初期処理
        /// </summary>
        private void InitPlclike()
        {
            // Plclike 起動フォームが未生成なので生成します。
            fPlc = new PlclikeClient.frmPLCSend();
            fPlc.dgtTransferOpenClose += ChangeTransferOpenClose;
            fPlc.dgtTransferMemory += ChangePLCData;
            fPlc.Disposed += ChangeTransferDispose;
        }

        /// <summary>
        /// PlclikeClient 起動
        /// </summary>
        private bool StartPlclike()
        {
            bool bRes = false;
            string message = "";

            // 処理内容へ表示
            message = "PlclikeClient 起動開始";
            AddLoggingMessage(message);

            // Plclike 起動フォーム生成判定
            if (fPlc == null)
            {
                // Plclike 起動フォームが未生成なので生成します。
                InitPlclike();
            }
            else if (fPlc.IsDisposed == true)
            {
                // Plclike 起動フォームが未生成なので生成します。
                InitPlclike();
            }
            else
            {
                message = "PlclikeClient はすでに実行中です";
                AddLoggingMessage(message);
                if(this.radioButton1.Checked == true)
                {
                    if( fPlc.Visible == false)
                    {
                        message = "PlclikeClient はすでに実行中ですが、PlclikeClient は非表示です。\r\n";
                        message += "PlilikeClient を表示しますか？";
                        if( MessageBox.Show(message,"起動確認", MessageBoxButtons.YesNo
                            , MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                        {
                            fPlc.Visible = true;
                        }
                        return true;
                    }
                }
                else
                {
                    if (fPlc.Visible == true)
                    {
                        message = "PlclikeClient はすでに実行中です。\r\n";
                        message += "PlilikeClient を非表示しますか？";
                        if (MessageBox.Show(message, "起動確認", MessageBoxButtons.YesNo
                            , MessageBoxIcon.Exclamation, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                        {
                            fPlc.Visible = false;
                        }
                        return true;
                    }
                }
                return true;
            }

            // Plclike 起動フォーム実行判定
            if (this.radioButton1.Checked == true)
            {
                // PlclikeClient 起動フォームを表示して実行
                message = "PlclikeClient 起動画面を表示して実行";
                AddLoggingMessage(message);
                fPlc.Show();
            }
            else
            {
                // PlcLikeClient 起動フォームを非表示で実行
                message = "PlclikeClient 起動画面を非表示して実行";
                AddLoggingMessage(message);
                fPlc.ShowHide();
            }

            // PlclikeClient ライセンス確認
            if (fPlc.LicenseFlag == false)
            {
                if (MessageBox.Show("ライセンス認証されていないようです・・・\r\n" +
                    "ライセンス認証されていない場合、設定ファイル等は使用できませんので設定する必要があります。\r\n" +
                    "設定する場合は「はい」、中断する場合は「いいえ」を選択してください", "ライセンス確認",
                    MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2) == DialogResult.No)
                {
                    // 処理の中断
                    return false;
                }


                // PlclikeClient 通信設定を実行
                bRes = ConfigPlclike();
                if( bRes == false)
                {
                    return false;
                }
            }

            // 設定情報取得
            SetCommboBoxDevice(true);
            this.textAddress.Text = "0000";
            this.textValue.Text = "";
            this.textAddress2.Text = "0000";
            this.textValue2.Text = "0000";
            return true;
        }

        /// <summary>
        /// PlclikeClient設定情報起動
        /// </summary>
        /// <returns></returns>
        private bool ConfigPlclike()
        {
            bool bRes = false;
            string message = "";

            // 処理内容へ表示
            message = "PlclikeClient 通信設定開始";
            AddLoggingMessage(message);

            // Plclike 起動フォーム生成判定
            if (fPlc == null)
            {
                // Plclike 起動フォームが未生成なので生成します。
                InitPlclike();
            }
            else if (fPlc.IsDisposed == true)
            {
                // Plclike 起動フォームが未生成なので生成します。
                InitPlclike();
            }
            // Plclike 通信設定を実行
            bRes = fPlc.StartTransferConfig();
            if (bRes == false)
            {
                MessageBox.Show("通信設定をキャンセル、または設定失敗だったので処理を中断します・・・",
                    "設定確認", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return false;
            }
            return true;
        }

        /// <summary>
        /// デバイス名称生成
        /// </summary>
        /// <param name="default_value">既定値使用区分(true=設定,false=未設定)</param>
        private void SetCommboBoxDevice(bool default_value)
        {
            int iMax = -1;
            int iLp = -1;
            string message = "";
            string value = "";

            // PLC通信情報の有無
            if (fPlc == null)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }
            else if (fPlc.IsDisposed == true)
            {
                // PLC通信起動確認警告
                message = "PLC通信情報が実行されていません。まず、「実行」ボタンをクリックしPLC通信を起動してください。";
                AddLoggingMessage(message);
                return;
            }

            // 値設定初期化
            fPlc.CreateComboDeviceRead(this.comboDevice);

            fPlc.CreateComboDeviceWrite(this.comboDevice2);

            if(default_value==true)
            {
                if ( this.comboDevice.Items.Count>=1)
                {
                    this.comboDevice.SelectedIndex = 0;
                }
                if (this.comboDevice2.Items.Count >= 1)
                {
                    this.comboDevice2.SelectedIndex = 0;
                }
            }
        }
        
        #region"＞処理内容表示処理"

        /// <summary>
        /// 処理内容メッセージ初期化
        /// </summary>
        private void ClrLoggingMessage()
        {
            this.textPlclike.Text = "";
        }

        /// <summary>
        /// 処理内容メッセージ追記
        /// </summary>
        /// <param name="value">メッセージ文字列</param>
        private void AddLoggingMessage(string value)
        {
            // 既定は、先頭へメッセージ文字列を追記します。は
            AddLoggingessage(value, 0, true);
        }

        /// <summary>
        /// 処理内容メッセージ追記
        /// </summary>
        /// <param name="value">メッセージ文字列</param>
        /// <param name="top_or_bottom">メッセージ追記区分(0=先頭に追記,1=終端へ追記)</param>
        /// <param name="disp_time">処理内容に日時付加有無(true=日時を追加,false=日時を非表示)</param>
        private void AddLoggingessage(string value, int top_or_bottom, bool disp_time)
        {
            // メッセージ指定確認
            if (String.IsNullOrEmpty(value) == true)
            {
                // メッセージが無い場合は処理を中断
                return;
            }
            string message = "";
            // 日時表記有無
            if( disp_time==true)
            {
                message = DateTime.Now.ToString("yyyy/MM/dd HH:mm:ss") + " ";
            }
            message += value;
            // メッセージ追加
            if (top_or_bottom == 1)
            {
                this.textPlclike.Text += "\r\n" + message;
            }
            else
            {
                this.textPlclike.Text = message + "\r\n" + this.textPlclike.Text;
            }
        }

        #endregion

        #region "＞イベント処理"

        /// <summary>
        /// PlclikeClient 起動/停止 切替イベント
        /// </summary>
        /// <param name="iSerialNo">登録番号</param>
        private void ChangeTransferOpenClose(int iSerialNo)
        {
            // Plclike 起動状態確認
            DispRunStatus(0);
        }

        /// <summary>
        /// PlclikeClient 終了時 イベント
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ChangeTransferDispose(object sender, EventArgs e)
        {
            // PlclikeClient 起動状態確認
            DispRunStatus(1);
        }

        /// <summary>
        /// 読取り・書込みイベント処理
        /// </summary>
        /// <param name="iSerialNo">シリアル番号</param>
        /// <param name="iMode">入力区分</param>
        /// <param name="sDevice">デバイス</param>
        /// <param name="sAddress">アドレス</param>
        /// <param name="sResponce">書込戻り値</param>
        /// <param name="sMemoryData">読取取得値</param>
        private void ChangePLCData(int iSerialNo,int iMode, string sDevice, string sAddress,string sResponce, string sMemoryData)
        {
            if (this.InvokeRequired == true)
            {
                PlclikeClient.frmPLCSend.SetTransferMemory d = new PlclikeClient.frmPLCSend.SetTransferMemory(ChangePLCDataSub);
                this.BeginInvoke(d, new object[] { iSerialNo, iMode, sDevice, sAddress, sResponce, sMemoryData });
            }
            else
            {
                ChangePLCDataSub(iSerialNo, iMode, sDevice, sAddress, sResponce, sMemoryData);
            }
        }

        /// <summary>
        /// 読取り・書込みイベント処理
        /// </summary>
        /// <param name="iSerialNo">シリアル番号</param>
        /// <param name="iMode">入力区分</param>
        /// <param name="sDevice">デバイス</param>
        /// <param name="sAddress">アドレス</param>
        /// <param name="sResponce">書込戻り値</param>
        /// <param name="sMemoryData">読取取得値</param>
        public void ChangePLCDataSub(int iSerialNo, int iMode, string sDevice, string sAddress, string sResponce, string sMemoryData)
        {
            string message = "";

            // 読出しイベント処理の場合にログ出力処理
            if (iMode == 0)
            {
                // イベント取得情報を代入
                message = string.Format(" PLC読出しイベント: SerialNo[{0}] Device[{1}] Address[{2}] Responce[{3}] DATA[{4}]", iSerialNo.ToString(), sDevice, sAddress, sResponce, sMemoryData);

                // 処理内容へ表示
                AddLoggingMessage(message);

                // 取得値をコントロールへ代入
                this.textValue.Text = sMemoryData;
            }
            // 書込みイベント処理の場合にログ出力処理
            if (iMode == 1)
            {
                // イベント取得情報を代入
                message = string.Format(" PLC書込みイベント: SerialNo[{0}] Device[{1}] Address[{2}] Responce[{3}] DATA[{4}]", iSerialNo.ToString(), sDevice, sAddress, sResponce, sMemoryData);

                // 処理内容へ表示
                AddLoggingMessage(message);
            }

        }


        #endregion

        #endregion

    }
}
